
#include <lf/LFlist.h>

LFnode::LFnode(LFobject *object,LFnode *previous,LFnode *next)
{
    m_previous=previous;
    m_next=next;
    m_object=object;
}


LFlist::LFlist() 
{
    m_list=LFNULL;
    m_listEnd=LFNULL;
    m_listSel=LFNULL;
    m_count=0;
}


LFlist::~LFlist()
{
    Clear();
} 


LFnode * LFlist::Append(LFobject *LFobj)
{
    LFnode *node=new LFnode(LFobj,m_listEnd);
    if(m_list==LFNULL){
        m_list=node;m_listEnd=node;m_listSel=node;
    }else 
        {
            m_listEnd->m_next=node;
        }
    m_listEnd=node;
    ++m_count;
    return m_listEnd;
}


LFnode * LFlist::Insert(long nItem,LFobject *LFobj)
{
    if(nItem >= m_count){nItem=m_count-1;return Append(LFobj);}
    ++m_count;
    LFnode  *nPrev,*nNext,*nodeTmp=m_list;
    for(long i=0;i < m_count;++i){
        if(i==nItem)break;
        nodeTmp=nodeTmp->m_next;
    }
    nPrev=nodeTmp->m_previous;
    nNext=nodeTmp;
    m_listSel=new LFnode(LFobj,nPrev,nNext);
    if(nPrev!=LFNULL){nPrev->m_next=m_listSel;} else {m_list=m_listSel;}
    nNext->m_previous=m_listSel;
    return m_listSel;
}


LFobject * LFlist::Delete(long nItem)
{
    if(nItem >= m_count)return false;

    LFnode   *nodeTmp=m_list;
    for(long i=0;i < m_count;++i){
        if(i==nItem){return DeleteNode(nodeTmp);}
        nodeTmp=nodeTmp->m_next;
    }
    return LFNULL;
}


LFobject * LFlist::Delete(LFnode *node)
{
    LFnode  *nodeTmp=m_list;
    for(long i=0;i < m_count;++i){
        if(nodeTmp==node){return DeleteNode(nodeTmp);}
        nodeTmp=nodeTmp->m_next;
    }
    return LFNULL;
}

LFobject * LFlist::Delete(LFobject *LFobj)
{
    LFnode  *nodeTmp=m_list;
    for(long i=0;i < m_count;++i){
        if(nodeTmp->m_object==LFobj){return DeleteNode(nodeTmp);}
        nodeTmp=nodeTmp->m_next;
    }
    return LFNULL;
}


LFobject * LFlist::DeleteNode(LFnode *node)
{
    LFobject *objRet=node->m_object;
    if(m_delobj)delete node->m_object;
    --m_count;
    if((node->m_next!=LFNULL)&&(node->m_previous!=LFNULL)){
        node->m_previous->m_next=node->m_next;
        node->m_next->m_previous=node->m_previous;
        delete node;
        return objRet;
    }
    if((node->m_next==LFNULL)&&(node->m_previous==LFNULL)){
        delete node;
        m_list=LFNULL;
        m_listEnd=LFNULL;
        m_listSel=LFNULL;
        return objRet;
    }
    if(node->m_previous==LFNULL){
        m_list=node->m_next;
        if(node==m_listSel){m_listSel=m_list;}
        delete node;
        return objRet;
    }
    if(node->m_next==LFNULL){
        m_listEnd=node->m_previous;
        if(node==m_listSel){m_listSel=m_listEnd;}
        delete node;
        return objRet;
    }
    return LFNULL;
}


void LFlist::Clear()
{
    LFnode  *nodeTmp=m_list;
    while(m_listEnd)
    {
        nodeTmp=m_listEnd;
        m_listEnd=m_listEnd->m_previous;
        delete nodeTmp;
    }
    m_listEnd=LFNULL;
    m_count=0;
}


LFnode * LFlist::GetFirst()
{
    m_listSel=m_list;
    return m_listSel;
}


LFnode * LFlist::GetLast()
{
    m_listSel=m_listEnd;
    return m_listSel;
}


LFnode * LFlist::GetPrevious(LFnode *node)
{
    if(node==LFNULL)node=m_listSel->m_previous;
        else node=node->m_previous;
    if(node!=LFNULL){m_listSel=node;}
    m_listSel=node;
    return node;
}


LFnode * LFlist::GetNext(LFnode *node)
{
    if(node==LFNULL)node=m_listSel->m_next;
        else node=node->m_next;
    if(node!=LFNULL){m_listSel=node;}
    return node;
}


LFobject * LFlist::GetNodeObject(LFnode *node)
{
    return node->m_object;
}


long LFlist::GetCount()
{
    return m_count;
}


LFnode * LFlist::FindObject(LFobject *LFobj)
{
    LFnode  *nodeTmp=m_list;
    for(long i=0;i < m_count;++i){
        if(nodeTmp->m_object==LFobj){DeleteNode(nodeTmp);return nodeTmp;}
        nodeTmp=nodeTmp->m_next;
    }
    return LFNULL;
}

