#include <lf/LFstring.h>

#include <cstdio>
#include <cstring>
using namespace std; 


LFstring::LFstring()
{
    initLFstring(LFNULL,LFSTRLENGTHMIN,true);
}


LFstring::LFstring(const LFstring &LFstr)
{

    initLFstring(LFNULL);
    Copy(LFstr);
}


LFstring::LFstring(const char *value,long length,bool check)
{
    initLFstring(value,length,check);
} 

 
void LFstring::initLFstring(const char *value,long length,bool check)
{
    m_check=check;
    m_buffer=LFNULL;
    m_length=0;
    MemAlloc(length);
    if(value!=LFNULL){
        Copy(value);
    }
} 


LFstring::~LFstring()
{
    if(m_buffer!=LFNULL)delete m_buffer; 
}

 
const LFstring& LFstring::operator = (const char * value)
{
    Copy(value);
    return *this;
}


const LFstring& LFstring::operator = (const LFstring& value)
{ 
    Copy(value); 
    return *this;
}


LFstring LFstring::operator + (const LFstring& value)
{
    Append(value); 
    LFstring strTmp=*this;
    return strTmp;
}


LFstring operator + (const char * value,const LFstring& value2)
{
    LFstring strTmp(value);
    strTmp.Append(value2); 
    return strTmp;
}


LFstring LFstring::operator + (const char * value)
{
    Append(value); 
    LFstring strTmp(*this);
    return strTmp;
}


LFstring& LFstring::operator << (const long value)
{
    char numTmp[20];
    sprintf(numTmp,"%d",(int)value);
    Append(numTmp);
    return *this;
}


LFstring& LFstring::operator << (const char * value)
{
    Append(value);
    return *this;
}


LFstring& LFstring::operator << (const LFstring& value)
{
    if( &value == this)
    {
        LFstring tmp(value);
        Append(tmp.c_str());
    }else{
        Append(value.c_str());
    }
    return *this;
}


bool LFstring::operator == (const LFstring& value)
{
    if(strcmp(c_str(),value.c_str())==0)return true;else return false;
}


bool LFstring::operator == (const char * value)
{
    if(strcmp(c_str(),value)==0)return true;else return false;
}


bool LFstring::operator != (const LFstring& value)
{
    if(strcmp(c_str(),value.c_str())!=0)return true;else return false;
}


bool LFstring::operator != (const char * value)
{
    if(strcmp(c_str(),value)!=0)return true;else return false;
}


char& LFstring::operator [] (long  value)
{
    if(m_check){
        if(value >= m_length)return m_buffer[m_length];
    }
    return m_buffer[value];
}


char * LFstring::nc_str()
{
    return m_buffer; 
}


const char * LFstring::c_str() const
{
    return m_buffer; 
}


void  LFstring::Append(const char *source)
{
    long tmpSize;
    if(m_check){
        tmpSize = strlen(m_buffer) + strlen(source);
        if((tmpSize) >= m_length){
            MemReAlloc(tmpSize);
        }
    }
    strcat(m_buffer,source);
}


void  LFstring::Append(const LFstring &LFstr)
{
    Append(LFstr.c_str());
}


void  LFstring::Copy(const LFstring &LFstr)
{
    Copy(LFstr.c_str());
}


void  LFstring::Copy(const char *source)
{
    long tmpSize;
    if(m_check){
        tmpSize = strlen(source);
        if(tmpSize >= m_length){
            MemReAlloc(tmpSize,false);
        }
    }
    strcpy(m_buffer,source); 
}


char * LFstring::Alloc(long size)
{
    MemReAlloc(size);
    return m_buffer;
}


void  LFstring::MemAlloc(long size)
{
    m_buffer=new char[size];m_buffer[0]=0;
    m_length=size;
}


void  LFstring::MemReAlloc(long size,bool copy)
{ 
    char *bufTmp=m_buffer;long tmpSize=m_length;
    MemAlloc(size + LFSTRLENGTHMIN);
    if(copy){memcpy(m_buffer,bufTmp,tmpSize);}
    delete [] bufTmp;
}

